/* global rootPath */
const inquirer = require('inquirer');
const memFs = require('mem-fs');
const editor = require('mem-fs-editor');
const { spawnSync } = require('child_process');
const chalk = require('chalk');
const fullname = require('fullname');
const R = require('ramda');

const store = memFs.create();
const fs = editor.create(store);
const log = console.log;

const dash = str => str.toLowerCase().replace(/ /g, '');

function installPackages() {
  log(`\nRunning ${chalk.bold('`npm install`')} for you..`);
  return spawnSync('npm', ['install'], {
    stdio: 'inherit',
    shell: true,
  });
}

function goToDirectory(directory) {
  try {
    return process.chdir(directory);
  } catch (err) {
    console.error(`chdir: ${err}`);
  }
}

function initGit() {
  log(`\nRunning ${chalk.bold('`git init`')} for you..`);
  return spawnSync('git', ['init'], {
    stdio: 'inherit',
  });
}

function gitLinkToRemote() {
  log(`Running ${chalk.bold('`git remote add origin`')} for you.`);
  return spawnSync('git', ['remote', 'add', 'origin', options.gitUrl], {
    stdio: 'inherit',
  });
}

function initWidget(widgetDir, options) {
  goToDirectory(widgetDir);

  installPackages();

  if (options.git) {
    initGit();
    if (options.gitUrl) {
      gitLinkToRemote();
    }
  }
  log(chalk.bold.green('\nYour widget is ready!'));
  log(
    `Type ${chalk.bold(`\`cd ${options.dashName}\``)} to go into your widget`,
    `then ${chalk.bold('`npm start`')} to preview your widget.\n`
  );
}

function createFiles(options, npmScope) {
  const dashName = dash(options.name);
  // options.packageName = `${npmScope}/${dashName}`;
  options.dashName = dashName;
  const widgetDir = `./${dashName}`;
  const templateDir = `${rootPath}/tasks/new/template`;

  log('Creating files for your new widget..');
  // Copy base files
  fs.copyTpl(`${templateDir}/base/**`, widgetDir, options);
  // Copy .gitignore
  if ((options.git = true)) {
    fs.copyTpl(`${templateDir}/gitignore`, `${widgetDir}/.gitignore`, options);
  }
  // Rollup, Babel, ESlint, Prettier
  fs.copyTpl(`${templateDir}/extra/**`, widgetDir, options);
  fs.copyTpl(`${templateDir}/extra/.*`, widgetDir, options);
  // Main js file
  fs.copyTpl(`${templateDir}/main.js`, `${widgetDir}/src/main.js`, options);
  // Form js file
  fs.copyTpl(`${templateDir}/form.js`, `${widgetDir}/src/form.js`, options);
  // Copy UI files
  if (options.ui) {
    fs.copyTpl(`${templateDir}/ui/**`, `${widgetDir}/src/`, options);
  }
  fs.commit(() => {
    log(chalk.green('Files created!'));
    initWidget(widgetDir, options);
  });
}

function thankYou(answers) {
  // execute('npm config get scope', function (stdout) {
  //   createFiles(answers, stdout);
  // });
  createFiles(answers);
}

function askQuestions(widgetName) {
  const questions = [
    {
      type: 'input',
      name: 'name',
      message: 'Widget name',
      default: widgetName,
      validate(answer) {
        if (answer.length < 1) {
          return 'You must name your widget.';
        }
        return true;
      },
    },
    {
      type: 'input',
      name: 'description',
      message: 'Widget description',
      default: 'A simple widget',
    },
    {
      type: 'input',
      name: 'author',
      message: 'Author',
      default() {
        return new Promise(resolve => {
          fullname().then(name => {
            resolve(name);
          });
        });
      },
    },
    {
      type: 'confirm',
      name: 'ui',
      message: 'Will your widget have an UI? (HTML & CSS)',
    },
    {
      type: 'confirm',
      name: 'git',
      message: 'Do you want to set up Git?',
    },
    {
      type: 'confirm',
      name: 'gitUrl',
      message: 'Do you already have a remote url for your repository?',
      default: false,
      when(answers) {
        return !!answers.git;
      },
    },

    // TODO:
    // Use ESLint to lint your code?

    // {
    //   type: 'confirm',
    //   name: 'es6',
    //   message: 'Do you want ES6?',
    // },
    // {
    //   type: 'confirm',
    //   name: 'lint',
    //   message: 'Do you want a linter?',
    // },
    // {
    //   type: 'confirm',
    //   name: 'uglify',
    //   message: 'Do you want a minifier?',
    // },
  ];

  // Do you want to use git?
  //

  // Do you want a module manager (rollup) with ES6 (Bublé), a linter (eslint), and a minifier (Uglyfy)?

  // Which JS preprocessor do you want?
  // ❯ ES2015 today with Babel
  //   Pure old JavaScript
  //   Typescript

  inquirer.prompt(questions).then(thankYou);
}

module.exports = askQuestions;
