/**
 * A controller to make the webpage interact the Electron UI.
 * @namespace webpageController
 */

/**
 * @external fs
 * @see {@link https://nodejs.org/api/fs.html}
 * @example
 * const fs = require('fs');
 * fs.readFileSync(filePath, 'utf8');
 * */
const fs = require('fs');

const { startSpinningAnimation, isValidUrl, addProtocolToUrl } = require('../helpers');
const events = require('./events');

const {
  $WEBVIEW__WEBPAGE,
  $SELECT__WEBPAGE_RESIZE,
  $INPUT__WEBPAGE_WIDTH,
  $INPUT__WEBPAGE_HEIGHT,
  $INPUT__WEBPAGE_ROTATE_PORTRAIT,
  $INPUT__WEBPAGE_ROTATE_LANDSCAPE,
  $SPAN__ROTATE_BUTTONS,
  $INPUT__WEBPAGE_URL,
  PORTRAIT,
  LANDSCAPE,
  ACTIVE,
  CUSTOM,
} = require('../constants');

/**
 * @function getWidth
 * @private
 * @memberof webpageController
 *
 * @description Returns the value of the 'width' input of the interface.
 * This input is related to the size of the webpage webview.
 *
 * @return {number} the width (unit: pixels)
 */
const getWidth = () => $INPUT__WEBPAGE_WIDTH.value;

/**
 * @function setWidth
 * @private
 * @memberof webpageController
 *
 * @description Set the value of the 'width' input of the interface.
 * This input is related to the size of the webpage webview.
 */
const setWidth = value => {
  $INPUT__WEBPAGE_WIDTH.value = value;
};

/**
 * @function getHeight
 * @private
 * @memberof webpageController
 *
 * @description Returns the value of the 'height' input of the interface.
 * This input is related to the size of the webpage webview.
 *
 * @return {number} the height (unit: pixels)
 */
const getHeight = () => $INPUT__WEBPAGE_HEIGHT.value;

/**
 * @function setHeight
 * @private
 * @memberof webpageController
 *
 * @description Set the value of the 'height' input of the interface.
 * This input is related to the size of the webpage webview.
 */
const setHeight = value => {
  $INPUT__WEBPAGE_HEIGHT.value = value;
};

/**
 * @function getSelectedSize
 * @private
 * @memberof webpageController
 *
 * @description Return the value of the 'device' dropdown,
 * used to resize the webpage.
 *
 * @return {Object} DOM element of the selected option (contains a 'value' property)
 */
const getSelectedSize = () => $SELECT__WEBPAGE_RESIZE.querySelector('option:checked');

/**
 * @function isLandscape
 * @private
 * @memberof webpageController
 *
 * @description Tells if the webpage is currently displayed in landscape format.
 *
 * @return {Boolean}
 */
const isLandscape = () => getWidth() > getHeight();

/**
 * @function isPortrait
 * @private
 * @memberof webpageController
 *
 * @description Tells if the webpage is currently displayed in portrait format.
 *
 * @return {Boolean}
 */
const isPortrait = () => getHeight() > getWidth();

/**
 * @function setActiveRotateButton
 * @private
 * @memberof webpageController
 *
 * @param {string} orientation - Can be 'portrait' or 'lanscape'
 *
 * @description Toggles the class of the rotate buttons to 'active' or not
 * depending on the orientation of the webpage.
 */
const setActiveRotateButton = orientation => {
  $INPUT__WEBPAGE_ROTATE_PORTRAIT.className = orientation === PORTRAIT ? ACTIVE : '';
  $INPUT__WEBPAGE_ROTATE_LANDSCAPE.className = orientation === LANDSCAPE ? ACTIVE : '';
};

/**
 * @function openDevTools
 * @public
 * @memberof webpageController
 *
 * @description Opens the chrome developer tools related the to webpage webview.
 */
function openDevTools() {
  $WEBVIEW__WEBPAGE.openDevTools();
}

/**
 * @function fillUrlInput
 * @public
 * @memberof webpageController
 *
 * @param {string} value
 *
 * @description Fills the 'webpage' url input with a new value
 */
function fillUrlInput(value) {
  $INPUT__WEBPAGE_URL.value = value;
}

/**
 * @function updateSize
 * @public
 * @memberof webpageController
 *
 * @param {object} [event] Event sent from the dom (like a 'click' or 'change' event)
 *
 * @description Update the width/height of the webpage depending on the values of
 * the width and height input fields.
 */
function updateSize(event) {
  if (event) {
    // if received an event instead of a function call, should reset the field
    $SELECT__WEBPAGE_RESIZE.children[0].selected = 'selected';
  }
  const newWidth = getWidth();
  const newHeight = getHeight();
  $WEBVIEW__WEBPAGE.style.width = newWidth ? `${newWidth}px` : '100%';
  $WEBVIEW__WEBPAGE.style.height = newHeight ? `${newHeight}px` : '100%';
}

/**
 * @function rotate
 * @public
 * @memberof webpageController
 *
 * @param {('portrait'|'landscape')} orientation Orientation of the webpage in the interface
 *
 * @description Rotates the webpage to match the provided orientation
 */
function rotate(orientation) {
  const width = getWidth();
  const height = getHeight();
  const changeToLandscape = isPortrait() && orientation === LANDSCAPE;
  const changeToPortrait = isLandscape() && orientation === PORTRAIT;
  if (changeToLandscape || changeToPortrait) {
    setWidth(height);
    setHeight(width);
    updateSize();
    setActiveRotateButton(orientation);
  }
}

/**
 * @function reload
 * @public
 * @memberof webpageController
 *
 * @description Reload the webpage webview entirely (for exemple, after the url
 * input has been changed).
 */
function reload() {
  startSpinningAnimation(document.querySelector('#refresh'));
  $WEBVIEW__WEBPAGE.reload();
}

/**
 * @function injectWidget
 * @public
 * @memberof webpageController
 *
 * @description Inject any widget's source code into the webpage, along with the
 * data from the config form of the widget, which are mandatory for the widget
 * to work.
 *
 * @param {string} mainFilePath Relative path to the widget main script.
 * @param {string} widgetPackageName Name of the widget, extracted from its package.json file.
 * @param {object} data Object generated from the config form, contains
 * information that the main needs to be executed.
 */
function injectWidget(mainFilePath, widgetPackageName, data) {
  // 1 - Applies the json generated from the config form
  // TEST_ID set to 1 for testing purpose in dev context.
  $WEBVIEW__WEBPAGE.executeJavaScript(
    `var DATA = ${JSON.stringify(data)}, TEST_ID = 1, PACKAGE = '${widgetPackageName}';`
  );
  // 2 - Applies the widget into the webpage webview
  $WEBVIEW__WEBPAGE.executeJavaScript(fs.readFileSync(mainFilePath, 'utf8'));
}

/**
 * @function resetSizeInputs
 * @public
 * @memberof webpageController
 *
 * @description Reset the values of the 'width' and 'height' input to match those
 * of the current webpage when the selected size is custom, to keep coherent values
 * with what is displayed.
 */
function resetSizeInputs() {
  if (getSelectedSize().value === CUSTOM) {
    setWidth($WEBVIEW__WEBPAGE.offsetWidth);
    setHeight($WEBVIEW__WEBPAGE.offsetHeight);
  }
}

/**
 * @function resizeWebpageByDevice
 * @public
 * @memberof webpageController
 *
 * @description If a specific device is selected, this method updates the webpage
 * so that its width, height, and user agent match those of the selected device.
 * (Please note that selecting any device will reset the orientation of the webpage
 * into 'portrait' mode)
 */
function resizeWebpageByDevice() {
  const optionSelected = getSelectedSize();
  $SPAN__ROTATE_BUTTONS.style.display = 'none';
  if (optionSelected.value !== CUSTOM) {
    $SPAN__ROTATE_BUTTONS.style.display = 'inline';
  }
  setWidth(optionSelected.getAttribute('data-w'));
  setHeight(optionSelected.getAttribute('data-h'));
  $INPUT__WEBPAGE_HEIGHT.ua = optionSelected.getAttribute('data-ua');
  $WEBVIEW__WEBPAGE.loadURL($INPUT__WEBPAGE_URL.value, { userAgent: $INPUT__WEBPAGE_HEIGHT.ua });
  updateSize();
  setActiveRotateButton('portrait'); // all devices are initialized as 'portrait'
}

/**
 * @function prev
 * @public
 * @memberof webpageController
 *
 * @description Acts as an history manager. Reverse the webpage to its previous state.
 */
function prev() {
  $WEBVIEW__WEBPAGE.goBack();
}

/**
 * @function next
 * @public
 * @memberof webpageController
 *
 * @description Acts as an history manager. Reverse the webpage to its newer state.
 */
function next() {
  $WEBVIEW__WEBPAGE.goForward();
}

/**
 * @function redirect
 * @public
 * @memberof webpageController
 *
 * @param {string} [url] Url OR Google Search Term to redirect the webpage to.
 *
 * @description Redirect the webpage to a new url. If the value provided is not
 * a valid url, it will rigger a google search instead.
 *
 */
function redirect(url) {
  $WEBVIEW__WEBPAGE.src = isValidUrl(url)
    ? addProtocolToUrl(url)
    : `http://www.google.com/search?q=${url}`;
}

module.exports = {
  updateSize,
  rotate,
  reload,
  injectWidget,
  resizeWebpageByDevice,
  fillUrlInput,
  openDevTools,
  prev,
  next,
  redirect,
  resetSizeInputs,
  // event listeners
  events,
};
